@@ -8,7 +8,6 @@
 #include <hilti/ast/expressions/ctor.h>
 #include <hilti/ast/expressions/deferred.h>
 #include <hilti/ast/expressions/grouping.h>
-#include <hilti/ast/expressions/id.h>
 #include <hilti/ast/expressions/keyword.h>
 #include <hilti/ast/expressions/list-comprehension.h>
 #include <hilti/ast/expressions/logical-and.h>
@@ -16,6 +15,7 @@
 #include <hilti/ast/expressions/logical-or.h>
 #include <hilti/ast/expressions/member.h>
 #include <hilti/ast/expressions/move.h>
+#include <hilti/ast/expressions/name.h>
 #include <hilti/ast/expressions/pending-coerced.h>
 #include <hilti/ast/expressions/resolved-operator.h>
 #include <hilti/ast/expressions/ternary.h>
