@@ -20,10 +20,7 @@ namespace hilti::detail::cxx {
  * HILTI's linker.
  *
  * It's not *really* a linker, it's a component that adds additional C++ code
- * requires knowledge across all compilation units. That knowledge is
- * included with each compiled C++ code unit as JSON data inside comments.
- * The linker extracts all this information and then generates an additional
- * C++ code unit with corresponding global code.
+ * requires knowledge across all compilation units.
  */
 class Linker {
 public:
@@ -31,11 +28,11 @@ public:
 
     void add(const linker::MetaData& md);
     void finalize();
-    Result<cxx::Unit> linkerUnit(); // only after finalize and at least one module
+    Result<std::shared_ptr<cxx::Unit>> linkerUnit(); // only after finalize and at least one module
 
 private:
     CodeGen* _codegen;
-    std::optional<cxx::Unit> _linker_unit;
+    std::shared_ptr<cxx::Unit> _linker_unit;
 
     std::set<std::pair<std::string, std::string>> _modules;
     std::map<std::string, std::vector<cxx::linker::Join>> _joins;
