@@ -29,9 +29,9 @@
 using namespace hilti;
 using namespace hilti::util;
 
-void detail::__internal_error(const std::string& s) { logger().internalError(s); }
+void util::detail::internalError(const std::string& s) { logger().internalError(s); }
 
-void util::cannot_be_reached() { hilti::logger().internalError("code is executing that should not be reachable"); }
+void util::cannotBeReached() { hilti::logger().internalError("code is executing that should not be reachable"); }
 
 std::vector<std::string> util::split(std::string s, const std::string& delim) {
     std::vector<std::string> l;
@@ -178,7 +178,7 @@ hilti::rt::filesystem::path util::currentExecutable() {
     return normalizePath(exe);
 }
 
-void util::abort_with_backtrace() {
+void util::abortWithBacktrace() {
     std::cerr << "\n--- Aborting" << '\n';
     auto bt = hilti::rt::Backtrace().backtrace();
     for ( const auto& f : *bt )
@@ -192,50 +192,52 @@ double util::currentTime() {
     return static_cast<double>(tv.tv_sec) + static_cast<double>(tv.tv_usec) / 1e6;
 }
 
-std::string util::toIdentifier(const std::string& s, bool ensure_non_keyword) {
-    static char const* const hex = "0123456789abcdef";
-
+std::string util::toIdentifier(std::string s) {
     if ( s.empty() )
         return s;
 
-    std::string normalized = s;
+    if ( ! isdigit(s[0]) && std::all_of(s.begin(), s.end(), [](auto c) { return std::isalnum(c) || c == '_'; }) )
+        // Fast-path: no special-characters, no leading digits.
+        return s;
 
-    normalized = replace(normalized, "::", "_");
-    normalized = replace(normalized, ":", "_");
-    normalized = replace(normalized, ">", "_");
-    normalized = replace(normalized, ",", "_");
-    normalized = replace(normalized, ".", "_");
-    normalized = replace(normalized, " ", "_");
-    normalized = replace(normalized, "-", "_");
-    normalized = replace(normalized, "'", "_");
-    normalized = replace(normalized, "\"", "_");
-    normalized = replace(normalized, "__", "_");
+    auto buffer_size = s.size() * 3 + 1;
+    char* buffer = reinterpret_cast<char*>(alloca(buffer_size)); // max possible size of modified string
+    char* p = buffer;
 
-    while ( ::util::endsWith(normalized, "_") )
-        normalized = normalized.substr(0, normalized.size() - 1);
+    if ( isdigit(s[0]) )
+        // do not start with a digit
+        *p++ = '_';
 
-    std::string ns;
+    for ( auto c : s ) {
+        switch ( c ) {
+            case ':':
+            case '<':
+            case '>':
+            case ',':
+            case '.':
+            case ' ':
+            case '-':
+            case '\'':
+            case '"':
+            case '%': *p++ = '_'; break;
 
-    for ( auto c : normalized ) {
-        if ( isalnum(c) || c == '_' ) {
-            ns += c;
-            continue;
+            default: {
+                if ( std::isalnum(c) || c == '_' )
+                    *p++ = c;
+                else {
+                    static char const* const hex = "0123456789abcdef";
+                    *p++ = 'x';
+                    *p++ = hex[c >> 4U];
+                    *p++ = hex[c % 0x0f];
+                }
+
+                break;
+            }
         }
-
-        ns += "x";
-        ns += hex[c >> 4U];
-        ns += hex[c % 0x0f];
     }
 
-    ns = replace(ns, "__", "_");
-
-    if ( isdigit(ns[0]) )
-        ns = "_" + ns;
-
-    if ( ensure_non_keyword )
-        ns += "_";
-
-    return ns;
+    assert(p < buffer + buffer_size);
+    return std::string(buffer, p - buffer);
 }
 
 std::string util::prefixParts(const std::string& in, const std::string& prefix, const std::string& include_tag) {
