@@ -8,6 +8,7 @@
 #include <node/v8.h>
 #include <uv.h>
 
+#include "Executor.h"
 #include "IOLoop.h"
 #include "Types.h"
 #include "ZeekJS.h"
@@ -30,6 +31,7 @@ struct InitOptions {
   int thread_pool_size = 4;
   bool owns_process_state = false;
   bool owns_node_inspector = false;
+  int loop_timer_milliseconds = 10000;
 };
 
 // Class holding Node.js and V8 state.
@@ -37,7 +39,8 @@ class Instance {
  public:
   bool Init(plugin::Corelight_ZeekJS::Plugin* plugin, const InitOptions& options);
 
-  void BeforeExit();
+  void EmitProcessBeforeExit();
+  void EmitProcessExit();
   void Done();
 
   void SetZeekNotifier(plugin::Corelight_ZeekJS::IOLoop::PipeSource* n) {
@@ -47,9 +50,11 @@ class Instance {
   // Called by the thin JsLoopIOSource
   int GetLoopFd();
   void Process();
+  void ProcessLocked();
   void UpdateTime();
   double GetNextTimeout();
   bool IsAlive();
+  void StopLoopTimer();
 
   void SetJsCalled(bool js_called = true) { js_called_ = js_called; };
   bool WasJsCalled() { return js_called_; };
@@ -138,9 +143,12 @@ class Instance {
   std::unique_ptr<node::Environment, std::function<void(node::Environment*)>>
       node_environment_;
   uv_loop_t loop;
+  uv_timer_t loop_timer;
 
   v8::Isolate* isolate_;
 
+  v8::Persistent<v8::Context> context_;
+
   // Wrapping.
   std::unique_ptr<ZeekValWrapper> zeek_val_wrapper_;
 
@@ -156,6 +164,8 @@ class Instance {
   // invocations. This is the object that Node.js uses for CheckImmediate and
   // RunTimers, so lets do that, too.
   v8::Global<v8::Object> process_obj_;
+
+  Executor executor;
 };
 
 class EventHandler : public plugin::Corelight_ZeekJS::Js::EventHandler {
