@@ -15,6 +15,7 @@
 #include <zeek/zeek-config.h>
 #endif
 
+#include <zeek/StmtEnums.h>
 #include <zeek/Val.h>
 
 namespace plugin::Corelight_ZeekJS::compat {
@@ -40,6 +41,12 @@ inline zeek::ValPtr IntervalVal_New(double x) {
 }
 #endif
 
+#if ZEEK_VERSION_NUMBER < 60200
+constexpr zeek::detail::StmtTag STMT_EXTERN = zeek::detail::STMT_ANY;
+#else
+constexpr zeek::detail::StmtTag STMT_EXTERN = zeek::detail::STMT_EXTERN;
+#endif
+
 }  // namespace plugin::Corelight_ZeekJS::compat
 
 // Avoid warnings about bro_int_t in Zeek 5.1. Provide zeek_int_t here for
