@@ -128,6 +128,9 @@ protected:
     ValPtr DoExec(Frame* f, Val* v, StmtFlowType& flow) override;
     bool IsPure() const override;
 
+    bool IsMinMaxConstruct() const;
+    StmtPtr ConvertToMinMaxConstruct();
+
     StmtPtr s1;
     StmtPtr s2;
 };
@@ -225,42 +228,6 @@ protected:
     std::vector<std::pair<ID*, int>> case_label_type_list;
 };
 
-// Helper class. Added for script optimization, but it makes sense
-// in terms of factoring even without.
-class AddDelStmt : public ExprStmt {
-public:
-    TraversalCode Traverse(TraversalCallback* cb) const override;
-
-    bool IsPure() const override;
-
-    // Optimization-related:
-    StmtPtr DoReduce(Reducer* c) override;
-    bool IsReduced(Reducer* c) const override;
-
-protected:
-    AddDelStmt(StmtTag t, ExprPtr arg_e);
-};
-
-class AddStmt final : public AddDelStmt {
-public:
-    explicit AddStmt(ExprPtr e);
-
-    ValPtr Exec(Frame* f, StmtFlowType& flow) override;
-
-    // Optimization-related:
-    StmtPtr Duplicate() override;
-};
-
-class DelStmt final : public AddDelStmt {
-public:
-    explicit DelStmt(ExprPtr e);
-
-    ValPtr Exec(Frame* f, StmtFlowType& flow) override;
-
-    // Optimization-related:
-    StmtPtr Duplicate() override;
-};
-
 class EventStmt final : public ExprStmt {
 public:
     explicit EventStmt(EventExprPtr e);
@@ -780,4 +747,24 @@ protected:
     int expected_len;
 };
 
+// Statement that calls a std::function. These can be added to a Func body
+// to directly all a C++ method.
+class StdFunctionStmt : public Stmt {
+public:
+    StdFunctionStmt(std::function<void(const zeek::Args&, StmtFlowType&)> f)
+        : Stmt(STMT_STD_FUNCTION), func(std::move(f)) {}
+
+    ValPtr Exec(Frame* f, StmtFlowType& flow) override;
+
+    StmtPtr Duplicate() override {
+        reporter->Error("Duplicate() on StdFunctionStmt not implemented");
+        return {zeek::NewRef{}, this};
+    }
+
+    TraversalCode Traverse(TraversalCallback* cb) const override { return TC_CONTINUE; }
+
+private:
+    std::function<void(const zeek::Args&, StmtFlowType&)> func;
+};
+
 } // namespace zeek::detail
