@@ -18,8 +18,6 @@ enum StmtTag {
     STMT_NEXT,
     STMT_BREAK,
     STMT_RETURN,
-    STMT_ADD,
-    STMT_DELETE,
     STMT_LIST,
     STMT_EVENT_BODY_LIST,
     STMT_INIT,
@@ -31,8 +29,9 @@ enum StmtTag {
     STMT_ZAM,           // a ZAM function body
     STMT_NULL,
     STMT_ASSERT,
-    STMT_EXTERN, // for custom Stmt subclasses provided by plugins
-#define NUM_STMTS (int(STMT_EXTERN) + 1)
+    STMT_EXTERN,       // for custom Stmt subclasses provided by plugins
+    STMT_STD_FUNCTION, // statements that call a std::function from c++
+#define NUM_STMTS (int(STMT_STD_FUNCTION) + 1)
 };
 
 enum StmtFlowType {
