@@ -236,6 +236,20 @@ public:
 
     TableValPtr GetRecordFields();
 
+    /**
+     * Renders the Val into JSON string representation. For record values
+     * contained anywhere in the Val, two arguments control the JSON result
+     * (they have no effect on other types):
+     *
+     * @param only_loggable  If true, skips any fields that don't have the &log
+     * attribute.
+     *
+     * @param re  The regular expression matcher, if given, is used to strip the
+     * first match on any record field name in the resulting output.  See the
+     * to_json() BiF for context.
+     *
+     * @return  JSON data representing the Val.
+     */
     StringValPtr ToJSON(bool only_loggable = false, RE_Matcher* re = nullptr);
 
     template<typename T>
@@ -564,6 +578,8 @@ public:
     StringValPtr Replace(RE_Matcher* re, const String& repl, bool do_all);
 
 protected:
+    unsigned int ComputeFootprint(std::unordered_set<const Val*>* analyzed_vals) const override;
+
     void ValDescribe(ODesc* d) const override;
     ValPtr DoClone(CloneState* state) override;
 
@@ -1711,7 +1727,6 @@ UNDERLYING_ACCESSOR_DEF(TypeVal, zeek::Type*, AsType)
 extern ValPtr check_and_promote(ValPtr v, const TypePtr& new_type, bool is_init,
                                 const detail::Location* expr_location = nullptr);
 
-extern bool same_val(const Val* v1, const Val* v2);
 extern bool same_atomic_val(const Val* v1, const Val* v2);
 extern bool is_atomic_val(const Val* v);
 extern void describe_vals(const ValPList* vals, ODesc* d, int offset = 0);
