@@ -75,6 +75,12 @@ struct AnalyOpt {
     // Produce a profile of ZAM execution.
     bool profile_ZAM = false;
 
+    // ZAM profiling sampling rate. Set via ZEEK_ZAM_PROF_SAMPLING_RATE.
+    int profile_sampling_rate = 100;
+
+    // An associated file to which to write the profile.
+    FILE* profile_file = nullptr;
+
     // If true, dump out transformed code: the results of reducing
     // interpreted scripts, and, if optimize is set, of then optimizing
     // them.
@@ -217,6 +223,10 @@ extern void analyze_global_stmts(Stmt* stmts);
 // Returns the body and scope for the previously analyzed global statements.
 extern std::pair<StmtPtr, ScopePtr> get_global_stmts();
 
+// Informs script optimization that parsing is switching to the given module.
+// Used to associate module names with profiling information.
+extern void switch_to_module(const char* module);
+
 // Add a pattern to the "only_funcs" list.
 extern void add_func_analysis_pattern(AnalyOpt& opts, const char* pat);
 
@@ -238,6 +248,10 @@ extern void clear_script_analysis();
 // Called when Zeek is terminating.
 extern void finish_script_execution();
 
+// Returns true if the given call has a specialized ZAM equivalent when
+// used in a conditional.
+extern bool IsZAM_BuiltInCond(const CallExpr* c);
+
 // Used for C++-compiled scripts to signal their presence, by setting this
 // to a non-empty value.
 extern void (*CPP_init_hook)();
