@@ -4,10 +4,8 @@
 
 #include <getopt.h>
 
-#include <algorithm>
 #include <memory>
 #include <string>
-#include <type_traits>
 #include <utility>
 #include <vector>
 
@@ -16,11 +14,12 @@
 #include <hilti/ast/declarations/type.h>
 #include <hilti/compiler/init.h>
 
-#include <spicy/ast/detail/visitor.h>
 #include <spicy/ast/types/unit.h>
+#include <spicy/ast/visitor.h>
 #include <spicy/autogen/config.h>
 #include <spicy/compiler/init.h>
 
+#include "compiler/plugin.h"
 #include "config.h"
 #include "glue-compiler.h"
 
@@ -31,28 +30,44 @@ using Driver = ::zeek::spicy::Driver;
  * Visitor to type information from a HILTI AST. This extracts user-visible
  * types only, we skip any internal ones.
  */
-struct VisitorTypes : public hilti::visitor::PreOrder<void, VisitorTypes> {
-    explicit VisitorTypes(Driver* driver, hilti::ID module, hilti::rt::filesystem::path path, bool is_resolved)
-        : driver(driver), module(std::move(module)), path(std::move(path)), is_resolved(is_resolved) {}
+struct VisitorTypes : public spicy::visitor::PreOrder {
+    explicit VisitorTypes(Driver* driver, GlueCompiler* glue, bool is_resolved)
+        : driver(driver), glue(glue), is_resolved(is_resolved) {}
 
-    void operator()(const hilti::declaration::Type& t) {
-        assert(! t.type().typeID() || *t.type().typeID() == hilti::ID(module, t.id())); // ensure consistent IDs
+    void operator()(hilti::declaration::Module* n) final {
+        if ( n->uid().in_memory ) {
+            // Ignore modules built by us in memory.
+            module = {};
+            return;
+        }
+        module = n->scopeID();
+        path = n->uid().path;
 
-        if ( module == hilti::ID("hilti") || module == hilti::ID("spicy_rt") || module == hilti::ID("zeek_rt") )
+        if ( is_resolved && ! n->skipImplementation() )
+            glue->addSpicyModule(module, path);
+    }
+
+    void operator()(hilti::declaration::Type* n) final {
+        if ( module.empty() || module == hilti::ID("hilti") || module == hilti::ID("spicy_rt") ||
+             module == hilti::ID("zeek_rt") )
             return;
 
+        assert(! n->type()->type()->typeID() ||
+               n->type()->type()->typeID() == hilti::ID(module, n->id())); // ensure consistent IDs
+
         types.emplace_back(TypeInfo{
-            .id = hilti::ID(module, t.id()),
-            .type = t.type()._clone().as<hilti::Type>(),
-            .linkage = t.linkage(),
+            .id = hilti::ID(module, n->id()),
+            .type = n->type(),
+            .linkage = n->linkage(),
             .is_resolved = is_resolved,
             .module_id = module,
             .module_path = path,
-            .location = t.meta().location(),
+            .location = n->meta().location(),
         });
     }
 
     Driver* driver;
+    GlueCompiler* glue;
     hilti::ID module;
     hilti::rt::filesystem::path path;
     bool is_resolved;
@@ -61,8 +76,8 @@ struct VisitorTypes : public hilti::visitor::PreOrder<void, VisitorTypes> {
 
 Driver::Driver(std::unique_ptr<GlueCompiler> glue, const char* argv0, hilti::rt::filesystem::path lib_path,
                int zeek_version)
-    : ::spicy::Driver("<Spicy Plugin for Zeek>"), _glue(std::move(glue)) {
-    _glue->Init(this, zeek_version);
+    : ::spicy::Driver("<Spicy support for Zeek>"), _glue(std::move(glue)) {
+    _glue->init(this, zeek_version);
 
     ::spicy::Configuration::extendHiltiConfiguration();
     auto options = hiltiOptions();
@@ -195,6 +210,8 @@ hilti::Result<hilti::Nothing> Driver::compile() {
 
     SPICY_DEBUG("Running Spicy driver");
 
+    _error = hilti::Nothing();
+
     if ( auto x = ::spicy::Driver::compile(); ! x )
         return x.error();
 
@@ -243,23 +260,16 @@ std::vector<std::pair<TypeInfo, hilti::ID>> Driver::exportedTypes() const {
     return result;
 }
 
-void Driver::hookNewASTPreCompilation(std::shared_ptr<hilti::Unit> unit) {
-    if ( unit->extension() != ".spicy" )
-        return;
-
-    if ( unit->path().empty() )
-        // Ignore modules constructed in memory.
-        return;
-
-    auto v = VisitorTypes(this, unit->id(), unit->path(), false);
-    for ( auto i : v.walk(unit->module()) )
-        v.dispatch(i);
+void Driver::hookNewASTPreCompilation(const hilti::Plugin& plugin, hilti::ASTRoot* root) {
+    auto v = VisitorTypes(this, _glue.get(), false);
+    hilti::visitor::visit(v, root, ".spicy");
 
     for ( const auto& ti : v.types ) {
         SPICY_DEBUG(hilti::util::fmt("  Got type '%s' (pre-compile)", ti.id));
         _types[ti.id] = ti;
 
-        if ( auto et = ti.type.tryAs<hilti::type::Enum>(); et && ti.linkage == hilti::declaration::Linkage::Public ) {
+        if ( auto et = ti.type->type()->tryAs<hilti::type::Enum>();
+             et && ti.linkage == hilti::declaration::Linkage::Public ) {
             SPICY_DEBUG("    Automatically exporting public enum for backwards compatibility");
             _public_enums.push_back(ti);
         }
@@ -268,17 +278,17 @@ void Driver::hookNewASTPreCompilation(std::shared_ptr<hilti::Unit> unit) {
     }
 }
 
-void Driver::hookNewASTPostCompilation(std::shared_ptr<hilti::Unit> unit) {
-    if ( unit->extension() != ".spicy" )
-        return;
+bool Driver::hookNewASTPostCompilation(const hilti::Plugin& plugin, hilti::ASTRoot* root) {
+    if ( plugin.component != "Spicy" )
+        return false;
 
-    if ( unit->path().empty() )
-        // Ignore modules constructed in memory.
-        return;
+    if ( ! _need_glue )
+        return false;
 
-    auto v = VisitorTypes(this, unit->id(), unit->path(), true);
-    for ( auto i : v.walk(unit->module()) )
-        v.dispatch(i);
+    _need_glue = false;
+
+    auto v = VisitorTypes(this, _glue.get(), true);
+    hilti::visitor::visit(v, root, ".spicy");
 
     for ( auto&& t : v.types ) {
         SPICY_DEBUG(hilti::util::fmt("  Got type '%s' (post-compile)", t.id));
@@ -286,20 +296,13 @@ void Driver::hookNewASTPostCompilation(std::shared_ptr<hilti::Unit> unit) {
         hookNewType(t);
     }
 
-    _glue->addSpicyModule(unit->id(), unit->path());
+    if ( ! _glue->compile() )
+        _error = hilti::result::Error("glue compilation failed");
+
+    return true;
 }
 
-hilti::Result<hilti::Nothing> Driver::hookCompilationFinished(const hilti::Plugin& plugin) {
-    if ( ! _need_glue )
-        return hilti::Nothing();
-
-    _need_glue = false;
-
-    if ( _glue->compile() )
-        return hilti::Nothing();
-    else
-        return hilti::result::Error("glue compilation failed");
-}
+hilti::Result<hilti::Nothing> Driver::hookCompilationFinished(hilti::ASTRoot* root) { return _error; }
 
 void Driver::hookInitRuntime() { ::spicy::rt::init(); }
 
